package com.mortals.xhx.module.site.service.impl;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.matter.model.*;
import com.mortals.xhx.module.matter.service.MatterDatumFileService;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.site.model.SiteDatumLibraryQuery;
import com.mortals.xhx.module.site.model.SiteEntity;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.site.dao.SiteDatumLibraryDao;
import com.mortals.xhx.module.site.model.SiteDatumLibraryEntity;
import com.mortals.xhx.module.site.service.SiteDatumLibraryService;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
* SiteDatumLibraryService
* 站点材料公共库 service实现
*
* @author zxfei
* @date 2023-03-10
*/
@Service("siteDatumLibraryService")
public class SiteDatumLibraryServiceImpl extends AbstractCRUDServiceImpl<SiteDatumLibraryDao, SiteDatumLibraryEntity, Long> implements SiteDatumLibraryService {

    @Autowired
    private MatterService matterService;

    @Autowired
    private MatterDatumService matterDatumService;

    @Autowired
    private MatterDatumFileService matterDatumFileService;

    @Override
    protected void findAfter(SiteDatumLibraryEntity params, Context context, List<SiteDatumLibraryEntity> list) throws AppException {

        list.stream().peek(item->{
            List<MatterDatumFileEntity> matterDatumFileEntities = matterDatumFileService.find(new MatterDatumFileQuery().datumId(item.getId()));
            item.setMatterDatumFileList(matterDatumFileEntities);
        }).count();
        super.findAfter(params, context, list);
    }

    @Override
    protected void findAfter(SiteDatumLibraryEntity params, PageInfo pageInfo, Context context, List<SiteDatumLibraryEntity> list) throws AppException {
        list.stream().peek(item->{
            List<MatterDatumFileEntity> matterDatumFileEntities = matterDatumFileService.find(new MatterDatumFileQuery().datumId(item.getId()));
            item.setMatterDatumFileList(matterDatumFileEntities);
        }).count();
        super.findAfter(params, pageInfo,context, list);
    }

    @Override
    public Rest<String> addToMatterDatum(String libraryIds, Long matterId, Context context) {
        if (ObjectUtils.isEmpty(matterId)) {
            throw new AppException("请选择对应事项");
        }
        List<Long> libraryIdList = Arrays.asList(libraryIds.split(",")).stream().map(Long::parseLong).collect(Collectors.toList());
        SiteDatumLibraryQuery query = new SiteDatumLibraryQuery();
        query.setIdList(libraryIdList);
        List<SiteDatumLibraryEntity> entities = this.find(query);
        int success = 0;
        int fail = 0;
        List<MatterDatumEntity> addEntities = new ArrayList<>();

        for (SiteDatumLibraryEntity library : entities) {
            MatterDatumEntity datum = updateOrSave(library, matterId, context);
            //SiteMatterEntity siteMatterEntity = updateOrSave(matterEntity, siteId, context);
            if (!ObjectUtils.isEmpty(datum)) {
                addEntities.add(datum);
                success++;
            } else {
                fail++;
            }
        }
        if (!ObjectUtils.isEmpty(addEntities)) {
            matterDatumService.save(addEntities, context);
        }
        String msg = "当前加入材料已存在！";
        if (libraryIdList.size() == 1) {
            if (success > 0) {
                msg = "加入材料成功！";
            }
        } else if (libraryIdList.size() > 1) {
            if (success > 0 && fail == 0) {
                msg = String.format("加入材料成功%d条!", success);
            } else if (success > 0 && fail > 0) {
                msg = String.format("加入材料成功%d条，重复加入材料%d条！", success, fail);
            } else if (success == 0 && fail > 0) {
                msg = String.format("重复加入材料%d条！", fail);
            }
        }
        return Rest.ok(msg);
    }

    private MatterDatumEntity updateOrSave(SiteDatumLibraryEntity item, Long matterId, Context context) {
        if(item.getDatumId()==null){
            item.setDatumId(-1l);
        }
        MatterEntity matterEntity = matterService.get(matterId);
        if(ObjectUtils.isEmpty(matterEntity)){
            return null;
        }
        MatterDatumEntity matterDatumEntity = matterDatumService.get(item.getDatumId());
        if (ObjectUtils.isEmpty(matterDatumEntity)) {
            MatterDatumEntity datumEntity = BeanUtil.covert(item,MatterDatumEntity.class);
            datumEntity.setCreateTime(new Date());
            if(context!=null&&context.getUser()!=null) {
                datumEntity.setCreateUserId(context.getUser().getId());
            }
            datumEntity.setMatterName(matterEntity.getMatterName());
            datumEntity.setMatterId(matterId);
            datumEntity.setMatterCode(matterEntity.getMatterNo());
            return datumEntity;
        }
        return null;
    }
}