package com.mortals.xhx.module.app.web;

import cn.hutool.core.net.url.UrlBuilder;
import cn.hutool.core.util.ZipUtil;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.AppTypeEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.module.app.model.AppEntity;
import com.mortals.xhx.module.app.model.AppQuery;
import com.mortals.xhx.module.app.service.AppService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.model.SiteQuery;
import com.mortals.xhx.module.site.service.SiteService;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.Constant.CUSTAPP_ROOT_PATH;

/**
 * 自助终端应用
 *
 * @author zxfei
 * @date 2022-11-28
 */
@RestController
@RequestMapping("app")
public class AppController extends BaseCRUDJsonBodyMappingController<AppService, AppEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private SiteService siteService;

    public AppController() {
        super.setModuleDesc("自助终端应用");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "type", AppTypeEnum.getEnumMap());
        this.addDict(model, "shelves", paramService.getParamBySecondOrganize("App", "shelves"));
        this.addDict(model, "distribute", YesNoEnum.getEnumMap());
        this.addDict(model, "dateUpdate", YesNoEnum.getEnumMap());
        super.init(model, context);
    }

    @Override
    protected int infoAfter(Long id, Map<String, Object> model, AppEntity entity, Context context) throws AppException {
        List<AppEntity> appEntityList = this.service.find(new AppQuery().appCode(entity.getAppCode()), context);
        entity.setApplianceSiteScope(appEntityList.size());
        entity.setSiteIdList(appEntityList.stream().map(AppEntity::getSiteId).collect(Collectors.toList()));
        SiteEntity siteEntity = siteService.getCache(entity.getSiteId().toString());
        if(!ObjectUtils.isEmpty(siteEntity)){
            //请求地址 http://domian/app/siteCode/appcode/html
            String domainUrl = GlobalSysInfo.getParamValue(Constant.PARAM_SERVER_HTTP_URL, "http://192.168.0.98:11078");
            entity.setCustUrl(UrlBuilder.of(domainUrl)
                    .addPath(CUSTAPP_ROOT_PATH)
                    .addPath(siteEntity.getSiteCode())
                    .addPath(entity.getAppCode())
                    .addPath(entity.getVersion().toString())
                    .toString());
        }else{
            entity.setCustUrl("");
        }
        return super.infoAfter(id, model, entity, context);
    }

    /**
     * app应用部署
     */
    @PostMapping(value = "appDistribute")
    public Rest<String> appDistribute(@RequestBody AppEntity appEntity) {
        String busiDesc = this.getModuleDesc() + "自助服务应用部署";
        Rest<String> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            if (ObjectUtils.isEmpty(appEntity.getId())) {
                throw new AppException("应用Id不能为空！");
            }
            appEntity = this.service.get(appEntity.getId(), getContext());
            if (ObjectUtils.isEmpty(appEntity)) {
                throw new AppException("当前应用不存在！appId:" + appEntity.getId());
            }
            rest = this.service.appDistribute(appEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("自助服务应用部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }

    /**
     * app基础应用部署
     */
    @PostMapping(value = "appCommonDistribute")
    public Rest<Void> appCommonDistribute(@RequestBody AppEntity appEntity) {
        String busiDesc = this.getModuleDesc() + "自助服务应用部署";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            rest = this.service.appCommonDistribute(appEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("基础服务应用部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }


    /**
     * app应用克隆给其它站点
     */
    @PostMapping(value = "cloneAppsBySites")
    public Rest<Void> cloneAppsBySites(@RequestBody AppQuery appQuery) {
        String busiDesc = this.getModuleDesc() + "自助服务应用部署克隆";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            if (ObjectUtils.isEmpty(appQuery.getIdList())) {
                throw new AppException("应用appId列表不能为空！");
            }
            if (ObjectUtils.isEmpty(appQuery.getSiteIdList())) {
                throw new AppException("克隆站点Id列表不能为空！");
            }
            List<AppEntity> appEntityList = this.service.find(new AppQuery().idList(appQuery.getIdList()), getContext());
            List<SiteEntity> siteEntityList = siteService.find(new SiteQuery().idList(appQuery.getSiteIdList()), getContext());
            Rest<String> cloneRest = this.service.cloneAppsBySites(appEntityList, siteEntityList, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("自助服务应用部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }







    public static void main(String[] args) {


/*        String homeUrl = "app/1/2";
        String domainUrl = "http://192.168.0.98:11078";
        String s = UrlBuilder.of(domainUrl).addPath("app").addPath("1").addPath("2").toString();
        System.out.println(s);*/

        ZipUtil.unzip("F://1672973316144.zip","E://abc");

    }


}