package com.mortals.xhx.module.base.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.Context;
import com.mortals.framework.web.BaseJsonBodyController;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.matter.*;
import com.mortals.xhx.feign.matter.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("base/matter")
public class BaseMatterController extends BaseJsonBodyController {
    @Autowired
    private IMatterFeign iMatterFeign;
    @Autowired
    private IMatterDatumFeign iMatterDatumFeign;
    @Autowired
    private IMatterAcceptFeign iMatterAcceptFeign;
    @Autowired
    private IMatterChargesFeign iMatterChargesFeign;
    @Autowired
    private IMatterFlowlimitFeign iMatterFlowlimitFeign;
    @Autowired
    private IMatterIntermediaryFeign iMatterIntermediaryFeign;
    @Autowired
    private IMatterQuestionFeign iMatterQuestionFeign;
    @Autowired
    private IMatterSetbaseFeign iMatterSetbaseFeign;

    /**
     * 事项基本信息
     * @param id
     * @return
     */
    @GetMapping({"info"})
    @UnAuth
    public Rest<Object> info(@RequestParam(value = "id") Long id) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查看事项基本信息";

        int code;
        try {
            String result = iMatterFeign.info(id);
            JSONObject jsonObject = JSONObject.parseObject(result);
            MatterPdu pdu = JSONObject.parseObject(jsonObject.getString("data"),MatterPdu.class);
            code = jsonObject.getInteger("code");
            if(code > 0) {
                model.put("data", pdu);
                model.put("dict", jsonObject.get("dict"));
                model.put("message_info", busiDesc + "成功");
            }else {
                model.put("message_info", jsonObject.getString("msg"));
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    /**
     * 查看事项申请材料业务列表
     * @param query
     * @return
     */
    @PostMapping({"datum/list"})
    @UnAuth
    public Rest<Object> datumList(@RequestBody MatterDatumPdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查看事项申请材料业务列表";

        int code;
        try {
            Rest<RespData<List<MatterDatumPdu>>> result = iMatterDatumFeign.list(query);
            code = result.getCode();
            if(code > 0) {
                model.put("data", result.getData().getData());
                model.put("pageInfo", result.getData().getPageInfo());
                model.put("dict", result.getData().getDict());
                model.put("message_info", busiDesc + "成功");
            }else {
                model.put("message_info", result.getMsg());
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    /**
     * 事项受理条件
     * @param query
     * @return
     */
    @PostMapping({"accept/list"})
    @UnAuth
    public Rest<Object> acceptList(@RequestBody MatterAcceptPdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询事项受理条件";

        int code;
        try {
            Rest<RespData<List<MatterAcceptPdu>>> result = iMatterAcceptFeign.list(query);
            code = result.getCode();
            if(code > 0) {
                model.put("data", result.getData().getData());
                model.put("pageInfo", result.getData().getPageInfo());
                model.put("dict", result.getData().getDict());
                model.put("message_info", busiDesc + "成功");
            }else {
                model.put("message_info", result.getMsg());
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    /**
     * 查看事项办理流程列表
     * @param query
     * @return
     */
    @PostMapping({"flowlimit/list"})
    @UnAuth
    public Rest<Object> flowlimitList(@RequestBody MatterFlowlimitPdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查看事项办理流程列表";

        int code;
        try {
            Rest<RespData<List<MatterFlowlimitPdu>>> result = iMatterFlowlimitFeign.list(query);
            code = result.getCode();
            if(code > 0) {
                model.put("data", result.getData().getData());
                model.put("pageInfo", result.getData().getPageInfo());
                model.put("dict", result.getData().getDict());
                model.put("message_info", busiDesc + "成功");
            }else {
                model.put("message_info", result.getMsg());
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    /**
     * 查看事项收费标准列表
     * @param query
     * @return
     */
    @PostMapping({"charges/list"})
    @UnAuth
    public Rest<Object> chargesList(@RequestBody MatterChargesPdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查看事项收费标准列表";

        int code;
        try {
            Rest<RespData<List<MatterChargesPdu>>> result = iMatterChargesFeign.list(query);
            code = result.getCode();
            if(code > 0) {
                model.put("data", result.getData().getData());
                model.put("pageInfo", result.getData().getPageInfo());
                model.put("dict", result.getData().getDict());
                model.put("message_info", busiDesc + "成功");
            }else {
                model.put("message_info", result.getMsg());
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    /**
     * 查看事项设定依据列表
     * @param query
     * @return
     */
    @PostMapping({"setbase/list"})
    @UnAuth
    public Rest<Object> setbaseList(@RequestBody MatterSetbasePdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查看事项设定依据列表";

        int code;
        try {
            Rest<RespData<List<MatterSetbasePdu>>> result = iMatterSetbaseFeign.list(query);
            code = result.getCode();
            if(code > 0) {
                model.put("data", result.getData().getData());
                model.put("pageInfo", result.getData().getPageInfo());
                model.put("dict", result.getData().getDict());
                model.put("message_info", busiDesc + "成功");
            }else {
                model.put("message_info", result.getMsg());
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    /**
     * 查看事项中介服务列表
     * @param query
     * @return
     */
    @PostMapping({"intermediary/list"})
    @UnAuth
    public Rest<Object> intermediaryList(@RequestBody MatterIntermediaryPdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查看事项中介服务列表";

        int code;
        try {
            Rest<RespData<List<MatterIntermediaryPdu>>> result = iMatterIntermediaryFeign.list(query);
            code = result.getCode();
            if(code > 0) {
                model.put("data", result.getData().getData());
                model.put("pageInfo", result.getData().getPageInfo());
                model.put("dict", result.getData().getDict());
                model.put("message_info", busiDesc + "成功");
            }else {
                model.put("message_info", result.getMsg());
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    /**
     * 查看事项常见问题列表
     * @param query
     * @return
     */
    @PostMapping({"question/list"})
    @UnAuth
    public Rest<Object> questionList(@RequestBody MatterQuestionPdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查看事项常见问题列表";

        int code;
        try {
            Rest<RespData<List<MatterQuestionPdu>>> result = iMatterQuestionFeign.list(query);
            code = result.getCode();
            if(code > 0) {
                model.put("data", result.getData().getData());
                model.put("pageInfo", result.getData().getPageInfo());
                model.put("dict", result.getData().getDict());
                model.put("message_info", busiDesc + "成功");
            }else {
                model.put("message_info", result.getMsg());
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }
}
