package com.mortals.xhx.module.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.xhx.base.system.message.MessageService;
import com.mortals.xhx.busiz.req.ApiThirdPartyReq;
import com.mortals.xhx.busiz.req.BussinessThirdPartyReq;
import com.mortals.xhx.common.code.IsBusinessEnum;
import com.mortals.xhx.common.code.ThirdPartyStatusEnum;
import com.mortals.xhx.common.code.ThirdPartyTypeEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.business.dao.BusinessDao;
import com.mortals.xhx.module.business.model.BusinessEntity;
import com.mortals.xhx.module.business.model.BusinessQuery;
import com.mortals.xhx.module.business.service.BusinessService;
import com.mortals.xhx.module.site.model.SiteBusinessEntity;
import com.mortals.xhx.module.site.model.SiteBusinessQuery;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.service.SiteBusinessService;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.module.window.model.WindowBusinessEntity;
import com.mortals.xhx.module.window.model.WindowBusinessQuery;
import com.mortals.xhx.module.window.service.WindowBusinessService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_PHP_HTTP_URL;

/**
 * BusinessService
 * 站点业务 service实现
 *
 * @author zxfei
 * @date 2022-01-13
 */
@Service("businessService")
@Slf4j
public class BusinessServiceImpl extends AbstractCRUDCacheServiceImpl<BusinessDao, BusinessEntity, Long> implements BusinessService {
    @Autowired
    private SiteBusinessService siteBusinessService;
    @Autowired
    private WindowBusinessService windowBusinessService;
    @Autowired
    private SiteService siteService;
    @Autowired
    private MessageService messageService;


    @Override
    protected void findAfter(BusinessEntity params, PageInfo pageInfo, Context context, List<BusinessEntity> list) throws AppException {
        list.stream().peek(item -> {
            List<BusinessEntity> childs = this.find(new BusinessQuery().parentId(item.getId()));
            if (!ObjectUtils.isEmpty(childs)) {
                item.setChildren(childs);
            } else {
                item.setChildren(null);
            }
        }).count();
        super.findAfter(params, pageInfo, context, list);
    }

    @Override
    protected void saveBefore(BusinessEntity entity, Context context) throws AppException {

        super.saveBefore(entity, context);
    }

    @Override
    protected void updateBefore(BusinessEntity entity, Context context) throws AppException {

        super.updateBefore(entity, context);
    }


    @Override
    protected void updateAfter(BusinessEntity entity, Context context) throws AppException {
        //同步更新所有站点
        List<SiteBusinessEntity> collect = siteBusinessService.find(new SiteBusinessQuery().businessId(entity.getId())).stream()
                .map(item -> {
                    BeanUtils.copyProperties(entity, item, new String[]{"id"});
                    item.setBusinessName(entity.getName());
                    return item;
                }).collect(Collectors.toList());
        siteBusinessService.update(collect, context);

        //更新事项业务表中字段
        List<SiteBusinessEntity> siteBusinessEntities = siteBusinessService.find(new SiteBusinessQuery().businessId(entity.getId()))
                .stream()
                .filter(item -> !item.getBusinessName().equals(entity.getName()))
                .peek(item -> item.setBusinessName(entity.getName()))
                .collect(Collectors.toList());
        if (!ObjectUtils.isEmpty(siteBusinessEntities)) {
            siteBusinessService.update(siteBusinessEntities, context);
        }

        //通知第三方平台
        String phpUrl = GlobalSysInfo.getParamValue(PARAM_SERVER_PHP_HTTP_URL, "http://172.15.28.116:8090");
        phpUrl += "/inter/device/baseDataSave";
        BussinessThirdPartyReq bussinessThirdPartyReq = new BussinessThirdPartyReq();
        BeanUtils.copyProperties(entity, bussinessThirdPartyReq, BeanUtil.getNullPropertyNames(entity));
        ApiThirdPartyReq<BussinessThirdPartyReq> apiThirdPartyReq = new ApiThirdPartyReq<>();
        apiThirdPartyReq.setCode(YesNoEnum.YES.getValue());
        apiThirdPartyReq.setType(ThirdPartyTypeEnum.业务.getValue());
        apiThirdPartyReq.setStatus(ThirdPartyStatusEnum.修改.getValue());
        apiThirdPartyReq.setData(bussinessThirdPartyReq);
        messageService.sendThirdParty(phpUrl, JSON.toJSONString(apiThirdPartyReq));
        super.updateAfter(entity, context);
    }


    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {

        Set<Long> idSet = Arrays.asList(ids).stream().collect(Collectors.toSet());
        List<BusinessEntity> businessEntities = this.get(ids, context);
        //有子节点 禁止删除
        List<BusinessEntity> collect = businessEntities.stream().filter(f -> IsBusinessEnum.一级业务.getValue() == f.getIsBusiness()).collect(Collectors.toList());
        if (!ObjectUtils.isEmpty(collect)) {
            //删除节点包含一级节点，查询删除的子节点 是否包含在所有
            for (BusinessEntity businessEntity : collect) {
                List<BusinessEntity> childList = this.find(new BusinessQuery().parentId(businessEntity.getId()));
                boolean bool = false;
                for (BusinessEntity child : childList) {
                    if (!idSet.contains(child.getId())) {
                        bool = true;
                        break;
                    }
                    if (bool) {
                        throw new AppException("删除的父业务存在子业务，不能删除");
                    }
                }
            }
        }

        String phpUrl = GlobalSysInfo.getParamValue(PARAM_SERVER_PHP_HTTP_URL, "http://172.15.28.116:8090");
        phpUrl += "/inter/device/baseDataSave";
        for (BusinessEntity entity : businessEntities) {
            BussinessThirdPartyReq bussinessThirdPartyReq = new BussinessThirdPartyReq();
            BeanUtils.copyProperties(entity, bussinessThirdPartyReq, BeanUtil.getNullPropertyNames(entity));
            ApiThirdPartyReq<BussinessThirdPartyReq> apiThirdPartyReq = new ApiThirdPartyReq<>();
            apiThirdPartyReq.setCode(YesNoEnum.YES.getValue());
            apiThirdPartyReq.setType(ThirdPartyTypeEnum.业务.getValue());
            apiThirdPartyReq.setStatus(ThirdPartyStatusEnum.删除.getValue());
            apiThirdPartyReq.setData(bussinessThirdPartyReq);
            messageService.sendThirdParty(phpUrl, JSON.toJSONString(apiThirdPartyReq));
        }


        super.removeBefore(ids, context);
    }


    /**
     * @param ids
     * @param context
     * @param result
     * @throws AppException
     */
    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        super.removeAfter(ids, context, result);
        if (!ObjectUtils.isEmpty(ids)) {
            //级联删除窗口业务
            WindowBusinessQuery windowBusinessQuery = new WindowBusinessQuery();
            windowBusinessQuery.setSiteBusinessIdList(Arrays.asList(ids));
            List<WindowBusinessEntity> windowBusinessEntities = windowBusinessService.find(windowBusinessQuery);
            log.info("remove windowBusinessEntities size:{}", windowBusinessEntities.size());
            if (!ObjectUtils.isEmpty(windowBusinessEntities)) {
                Long[] idList = windowBusinessEntities.stream().map(item -> item.getId()).toArray(Long[]::new);
                windowBusinessService.remove(idList, context);
            }
/*            SiteBusinessQuery siteBusinessQuery = new SiteBusinessQuery();
            siteBusinessQuery.setBusinessIdList(Arrays.asList(ids));
            List<SiteBusinessEntity> siteBusinessEntities = siteBusinessService.find(siteBusinessQuery);
            log.info("remove siteBusinessEntities size:{}", siteBusinessEntities.size());
            if (!ObjectUtils.isEmpty(siteBusinessEntities)) {
                Long[] idList = siteBusinessEntities.stream().map(item -> item.getId()).toArray(Long[]::new);
                siteBusinessService.remove(idList, context);
            }*/
        }
    }

    @Override
    public void addBusinessToSite(String businessIds, Long siteId, Context context) {

        if (ObjectUtils.isEmpty(siteId)) {
            throw new AppException("请选择对应站点");
        }
        List<Long> businessIdList = Arrays.asList(businessIds.split(",")).stream().map(Long::parseLong).collect(Collectors.toList());
        //先删除后再新增
        SiteBusinessQuery siteBusinessQuery = new SiteBusinessQuery();
        siteBusinessQuery.setSiteId(siteId);
        siteBusinessQuery.setBusinessIdList(businessIdList);

        Long[] ids = siteBusinessService.find(siteBusinessQuery, context).stream().map(item -> item.getId()).toArray(Long[]::new);
        if (!ObjectUtils.isEmpty(ids)) {
            siteBusinessService.remove(ids, context);
        }

        BusinessQuery businessQuery = new BusinessQuery();
        businessQuery.setIdList(businessIdList);
        this.find(businessQuery).stream()
                .forEach(item -> {
                    //更新或新增
                    updateOrSave(item, siteId, context);
                });
    }

    @Override
    public Result<BusinessEntity> flatList(BusinessEntity query, PageInfo pageInfo, Context context) {
        Result<BusinessEntity> result = this.dao.getList(query, pageInfo);
        return result;
    }

    private void updateOrSave(BusinessEntity item, Long siteId, Context context) {
        SiteBusinessEntity siteBusinessEntity = new SiteBusinessEntity();

       // BeanUtils.copyProperties(item, siteBusinessEntity, BeanUtil.getNullPropertyNames(item));
        SiteEntity siteEntity = siteService.get(siteId);
        if (ObjectUtils.isEmpty(siteEntity)) {
            throw new AppException("未找到相应站点！");
        }
        siteBusinessEntity.setSiteId(siteEntity.getId());
        siteBusinessEntity.setSiteName(siteEntity.getSiteName());
        siteBusinessEntity.setBusinessId(item.getId());
        siteBusinessEntity.setBusinessName(item.getName());
        siteBusinessEntity.setAncestors(item.getAncestors());
        siteBusinessEntity.setIsBusiness(item.getIsBusiness());
        siteBusinessEntity.setParentId(item.getParentId());
        siteBusinessEntity.setStatus(item.getStatus());
        siteBusinessEntity.setRemark(item.getRemark());
        siteBusinessEntity.setCanorder(item.getCanorder());
        siteBusinessEntity.setCantake(item.getCantake());
        siteBusinessEntity.setDatashow(item.getDatashow());
        siteBusinessEntity.setEnglishname(item.getEnglishname());
        siteBusinessEntity.setFlownum(item.getFlownum());
        siteBusinessEntity.setBusinessType(item.getBusinessType());
        siteBusinessEntity.setCreateUserId(context == null ? 1L : context.getUser() == null ? 1L : context.getUser().getId());
        siteBusinessEntity.setCreateTime(new Date());

        if (siteBusinessEntity.getIsBusiness() == IsBusinessEnum.二级业务.getValue()
                && !ObjectUtils.isEmpty(siteBusinessEntity.getParentId())) {
            SiteBusinessEntity tempSiteBusinessEntity = siteBusinessService.selectOne(new SiteBusinessQuery()
                    .siteId(siteEntity.getId())
                    .businessId(siteBusinessEntity.getParentId()));
            if (ObjectUtils.isEmpty(tempSiteBusinessEntity)) {
                BusinessEntity parentBusinessEntity = this.get(siteBusinessEntity.getParentId(), context);
                SiteBusinessEntity parentSiteBusinessEntity = new SiteBusinessEntity();
                BeanUtils.copyProperties(parentBusinessEntity, parentSiteBusinessEntity, BeanUtil.getNullPropertyNames(item));
                parentSiteBusinessEntity.setSiteId(siteEntity.getId());
                parentSiteBusinessEntity.setSiteName(siteEntity.getSiteName());
                parentSiteBusinessEntity.setBusinessId(parentBusinessEntity.getId());
                parentSiteBusinessEntity.setBusinessName(parentBusinessEntity.getName());
                parentSiteBusinessEntity.setId(null);
                parentSiteBusinessEntity.setCreateUserId(context == null ? 1L : context.getUser() == null ? 1L : context.getUser().getId());
                parentSiteBusinessEntity.setCreateTime(new Date());
                siteBusinessService.save(parentSiteBusinessEntity, context);
            }
        }
        siteBusinessService.save(siteBusinessEntity, context);

    }

    public static void main(String[] args) {
        SiteBusinessQuery siteBusinessQuery = new SiteBusinessQuery();
        SiteBusinessQuery siteBusinessEntity = new SiteBusinessQuery();
        siteBusinessEntity.setSiteId(null);
        ArrayList<SiteBusinessQuery> objects = new ArrayList<>();
        objects.add(siteBusinessEntity);
        siteBusinessQuery.setAndConditionList(objects);
        System.out.println(JSON.toJSONString(siteBusinessQuery));


    }
}