package com.mortals.xhx.module.dept.web;

import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.Context;
import com.mortals.framework.web.BaseJsonBodyController;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.dept.DeptPdu;
import com.mortals.xhx.common.pdu.matter.MatterPdu;
import com.mortals.xhx.feign.dept.IDeptFeign;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 部门
 *
 * @author zxfei
 * @date 2022-01-20
 */
@RestController
@RequestMapping("dept")
public class DeptController extends BaseJsonBodyController {

    @Autowired
    private IDeptFeign deptFeign;

    @PostMapping({"list"})
    @UnAuth
    public Rest<Object> list(@RequestBody DeptPdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询部门列表";

        int code;
        try {
            Rest<RespData<List<DeptPdu>>> result = deptFeign.list(query);
            code = result.getCode();
            if(code > 0) {
                model.put("data", result.getData().getData());
                model.put("pageInfo", result.getData().getPageInfo());
                model.put("dict", result.getData().getDict());
                model.put("message_info", busiDesc + "成功");
            }else {
                model.put("message_info", result.getMsg());
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    /**
     * 事项基本信息
     * @param id
     * @return
     */
    @GetMapping({"info"})
    @UnAuth
    public Rest<Object> info(@RequestParam(value = "id") Long id) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查看事项基本信息";

        int code;
        try {
            Rest<DeptPdu> result = deptFeign.info(id);
            code = result.getCode();
            if(code > 0) {
                model.put("data", result.getData());
                model.put("dict", result.getDict());
                model.put("message_info", busiDesc + "成功");
            }else {
                model.put("message_info", result.getMsg());
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }
}